/* xtdio.h: an external header file for the pattern matching routines
 * Copyright  July 12, 1999: Charles E. Campbell, Jr. -- see <Copyright> file
 */
#ifndef XTDIO_H
#define XTDIO_H

#ifdef __cplusplus
extern "C" {
#endif

/* --------------------------------------------------------------------------
 * Includes:
 */

#if defined(AIX)
# include <string.h>
# include <stdarg.h>
# include <stdlib.h>
# ifndef unix
#  define unix
# endif

#elif defined(apollo)
# include <string.h>
# include <stdarg.h>
# include <stdlib.h>

#elif defined(__BORLANDC__)
# define  msdos
# include <stdio.h>
# include <stdlib.h>
# include <stdarg.h>
# include <string.h>
# include <time.h>

#elif defined(__DECC) || defined(__alpha) || defined(__osf__)
# include <string.h>
# include <stdlib.h>
# include <stdarg.h>

#elif defined(__linux)
# include <string.h>
# include <stdlib.h>
# include <stdarg.h>
# include <string.h>
# include <time.h>
# if defined(__STRICT_ANSI__) && !defined(__USE_POSIX)
	 extern FILE *fdopen __P ((int, __const char *));
	 extern int fileno __P ((FILE*));
	 extern FILE* popen __P ((__const char*, __const char*));
	 extern int pclose __P ((FILE*));
# endif

#elif defined(MCH_AMIGA)
# include <string.h>
# include <stdlib.h>
# include <stdarg.h>

#elif defined(__MSDOS__)
# define msdos
# ifndef MSDOS
#  define MSDOS
# endif
# ifndef msdos
#  define msdos
# endif
# include <stdio.h>
# include <stdlib.h>
# include <stdarg.h>
# include <string.h>
# include <time.h>

#elif defined(sgi)
# ifdef __MALLOC_DBG_H__
#  include "mallocdbg2.h"
# else
#  include <strings.h>
#  include <stdlib.h>
# endif
# include <stdarg.h>

#elif defined(sun)
# include <string.h>
# include "xstdlib.h"
# include "varargs.h"

#elif defined(ultrix)
/* # define OldUltrix */
# ifdef OldUltrix
#  include <string.h>
#  include <stdlib.h>
#  include <varargs.h>
# else
#  include <strings.h>
#  include <stdlib.h>
#  include <stdarg.h>
# endif

#elif defined(vms)

# ifndef __STDIO_LOADED
#  ifndef oldvms
#   define oldvms 1
#  endif
# endif

# ifdef oldvms
#  include "xstdlib.h"
#  include "xvarargs.h"
#  define  defined(x) 0
#  define  strtok stptok

# else	/* new vms */
#  include <string.h>
#  include <stdlib.h>
#  include <stdarg.h>
#  define fgets(s,n,fp)			vmsfgets(s,n,fp)
#  define read(fp,buf,bufsiz)	vmsread(fp,buf,bufsiz)
#  ifndef __DESCRIP_LOADED
#   include <descrip.h>
    typedef struct dsc$descriptor DESC;
#  endif
# endif
# define system(cmd) vms_system(cmd)

#endif

#ifndef __STDIO_H__
# include <stdio.h>
#endif

#include "setproto.h"		/* sets __PROTOTYPE__ */

/* --------------------------------------------------------------------- */

/* standard Dr. Chip includes */
#ifndef NO_GETARG_H
# include "getarg.h"
#endif
#include "rdcolor.h"
#include "links.h"

#ifndef EXIT
# define EXIT(rc) exit(rc)
#endif

#ifdef ultrix
# define const
#endif

/* -------------------------------------------------------------------------
 * Definitions:
 */
#define isoct(c) ('0' <= (c) && (c) <= '7')
#define ishex(c) \
 (isdigit(c) || ('a' <= (c) && (c) <= 'f') || ('A' <= (c) && (c) <= 'F'))

/* error support */
#define XTDIO_SEVERE		0
#ifndef XTDIO_ERROR
# define XTDIO_ERROR		1
#endif
#ifndef XTDIO_WARNING
# define XTDIO_WARNING	2
#endif
#define XTDIO_NOTE		3
#define XTDIO_USAGE		4
#define XTDIO_ERRORMORE	5

#ifndef USE_ESCAPES
# define USE_ESCAPES		1
#endif

/* fopensave support */
extern char *fopenfilename;

/* stcmpma support */
#define MTCHMAX 10

/* ---------------------------------------------------------------------
 * Machine Dependent Defines:
 */
#ifdef MCH_AMIGA
# define system(cmd)			SyStem(cmd)
# define bzero(obj,nbytes)		memset(obj,0,nbytes)
# define bcopy(tgt,src,nbytes)	memcpy(tgt,src,nbytes)
#endif

#ifdef sun
# define const
#endif

/* -------------------------------------------------------------------------
 * Typedefs:
 */
typedef struct PS_LEAF_str		PS_LEAF;
typedef struct PS_SORT_str		PS_SORT;
typedef struct PS_str			PS;
typedef struct PS_str			ParSrch;

/* -------------------------------------------------------------------------
 * Data Structures:
 */

/* supports ps_make, ps_insert, ps_search, ps_delete, etc */
struct PS_LEAF_str {	/* PS_LEAF:											*/
	char     c;			/* character needed to transition to this leaf		*/
	void    *output;	/* this ptr is output as match if end-of-word		*/
	int      id;		/* for debugging -- helps to track leaves			*/
	PS_LEAF *left;		/* supports left-side fanout from leaf->prv			*/
	PS_LEAF *right;		/* supports right-side fanout from leaf->prv		*/
	PS_LEAF *nxt;		/* points to next ps_tree node fanout root			*/
	PS_LEAF *prv;		/* points to root of fanout's predecessor			*/
	};

struct PS_str {			/* PS:	ParSrch:									*/
	PS_LEAF *tree;		/* first character is hashed						*/
	PS_SORT *shd;		/* head of linked list of PS_SORT entries			*/
	PS_SORT *stl;		/* tail of linked list of PS_SORT entries			*/
	PS_SORT *current;	/* support for ps_srtnxt ps_srtprv					*/
	int      treesize;	/* starts at 128; may be boosted to 256 if needed	*/
	int      nwords;	/* qty of words stored in tree						*/
	int      id;		/* serves as identifier for this ParSrch			*/
#ifdef __PROTOTYPE__
	int    (*accept)(int);	/* this function restricts what characters		*/
#else	/* __PROTOTYPE__ */	/* are to be considered acceptable for ParSrch	*/
	int    (*accept)();		/* engine search.  Default: is isalnum()||'_'	*/
#endif	/* __PROTOTYPE__ */	/* note: isany() is pre-supplied				*/
	};

struct PS_SORT_str {	/* PS_SORT:											*/
	PS_LEAF *leaf;		/* associated PS_LEAF								*/
	PS_SORT *nxt;		/* doubly linked list support						*/
	PS_SORT *prv;		/* doubly linked list support						*/
	};

/* supports stcmpma */
struct mtchlst {		/* external sub-field match list					*/
	char *start;		/* the i'th sub-field start pointer					*/
	char *last;			/* the i'th sub-field last pointer					*/
	};

/* -------------------------------------------------------------------------
 * Global Variables:
 */

/* fopenv/fopenx support */
extern char *fopenvfile;
extern char *fopenxfile;

/* ncgets support */
extern char *ncgetsprompt;						/* allows prompt for no-comment-gets				*/

/* matdisp support */
extern char *matdispfmt;						/* default matdisp format (>= 5)					*/
extern char *imatdispfmt;						/* default matout format (>= 5)						*/
extern int   screenheight;						/* rows    in display								*/
extern int   screenwidth;						/* columns in a line available to matdisp			*/
extern int   use_escapes;						/* (don't) use ANSI escape sequences				*/

/* multicol support */
extern char  multicol_moreflag;					/* have multicol pause every screenheight-1 lines	*/
extern char *multicol_leader;					/* print out before every line						*/
extern char *multicol_oneleader;				/* print out before 1st line, blanks after			*/
extern char *multicol_section;					/* allow a section of text to be unaltered			*/
extern int   multicol_len;						/* maximum width of multicolumn output				*/
extern int   multicol_sectioncol;				/* begin output of section at/after this column		*/

extern unsigned long  autobuf_fgets_linenum;	/* autobuf_fgets line number						*/
extern char          *strnxtfmt_code;			/* strnxtfmt format-code pointer					*/

#ifdef __PROTOTYPE__
extern char *(*multicol_render)(char *,int);	/* multicol's rendering function					*/
#else
extern char *(*multicol_render)();
#endif

/* stcmpma support: */
extern struct mtchlst stcmpma_mtch[MTCHMAX];
extern int            stcmpma_mcnt;
extern char           stcmpma_underscore;

/* outofmem and error support */
#ifdef __PROTOTYPE__
extern void (*error_exit)(int);
#else
# define const
extern void (*error_exit)();
#endif

extern FILE *errfp;

/* getarg support */
#ifndef NO_GETARG_H
extern GA_Codes ga_codes[];
#endif

#ifdef __cplusplus
	}
#endif

/* ---------------------------------------------------------------------
 * Prototypes
 */
#include "proto.h"

#endif
